
# DXS

Design system based React UI component factory,
intended for x0, c8r/lab, and c8r/ui

```sh
npm i dxs
```

```js
// config.js
module.exports = [
  {
    name: 'Heading',
    type: 'h2',
    props: {
      m: 0,
      f: 5
    },
    style: {
      lineHeight: 1.25
    }
  },
  {
    name: 'Button',
    type: 'button',
    props: {
      f: 1,
      m: 0
    },
    style: props => ({
      fontFamily: 'inherit',
      display: 'inline-block',
      padding: '12px',
      border: 0,
      borderRadius: '3px',
      color: 'white',
      backgroundColor: props.theme.colors.blue,
      appearance: 'none',
      ':hover': {
        boxShadow: 'inset 0 0 0 999px rgba(0, 0, 0, .125)'
      }
    })
  },
  {
    // Button extension
    name: 'PillButton',
    type: 'Button',
    props: {},
    style: {
      borderRadius: '99999px'
    }
  }
]
```

```js
const dxs = require('dxs')
const config = require('./config')

const library = dxs(config)
// -> object of React UI components
```

- Uses [styled-system][system] for theme-based style props.
- Uses [cxs][cxs] for live components (configurable with `options.hoc`)
- Exports to file formats for [styled-components][sc] and [glamorous][glamorous]


[system]: https://github.com/jxnblk/styled-system
[cxs]: https://github.com/jxnblk/cxs
[sc]: https://github.com/styled-components/styled-components
[glamorous]: https://github.com/paypal/glamorous


## Exporting static modules

To export a configuration as an array of static JavaScript modules,
use the `modules` function.
This only is intended for use in node environments.

```js
const fs = require('fs')
const path = require('path')
const createModules = require('dxs/modules')
const config = require('./config')

const mods = createModules(config, {
  library: 'styled-components'
})

mods.forEach(mod => {
  const filename = path.join(__dirname, 'components', mod.name + '.js')
  fs.writeFileSync(filename, mod.module)
})
```

