'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var hash = require('./hash');
var sheet = require('./sheet');

/**
 *  cxs-like class
 *
 *  const createStyle = new CSS({
 *    prefix: 'ComponentName'
 *  })
 */

var CSS = function () {
  function CSS() {
    var _this = this;

    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, CSS);

    var _options$prefix = options.prefix,
        prefix = _options$prefix === undefined ? '_x' : _options$prefix,
        _options$id = options.id,
        id = _options$id === undefined ? '' : _options$id;


    this.cache = {};
    this.prefix = prefix;
    this.id = id;
    this.sheet = sheet;
    this.css = function () {
      return sheet.css();
    };

    this.reset = function () {
      _this.cache = {};
      sheet.reset();
    };

    var instance = Object.assign(this.createStyle.bind(this), this);

    return instance;
  }

  _createClass(CSS, [{
    key: 'parseRules',
    value: function parseRules(obj) {
      var child = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
      var media = arguments[2];

      var rules = [];
      var declarations = [];

      for (var key in obj) {
        var value = obj[key];

        if (value === null) continue;

        if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object') {
          var _media = /^@/.test(key) ? key : null;
          var _child = _media ? child : child + noAnd(key);
          this.parseRules(value, _child, _media).forEach(function (r) {
            return rules.push(r);
          });
          continue;
        }

        var dec = createDeclaration(key, value);
        declarations.push(dec);
      }

      rules.unshift({
        media: media,
        child: child,
        declarations: declarations
      });

      return rules;
    }
  }, {
    key: 'parse',
    value: function parse(obj) {
      var _this2 = this;

      var rules = this.parseRules(obj);
      var classNames = [];

      rules.forEach(function (rule) {
        var id = [_this2.prefix, _this2.id, '_', hash(JSON.stringify(rule))].join('');
        if (_this2.cache[id]) {
          classNames.push(_this2.cache[id]);
          return;
        }

        classNames.push(id);
        var ruleset = createRule(Object.assign(rule, { className: id }));

        sheet.insert(ruleset);
        _this2.cache[id] = id;
      });

      return classNames.join(' ');
    }
  }, {
    key: 'createStyle',
    value: function createStyle() {
      var _this3 = this;

      for (var _len = arguments.length, styles = Array(_len), _key = 0; _key < _len; _key++) {
        styles[_key] = arguments[_key];
      }

      return styles.map(function (style) {
        return _this3.parse(style);
      }).join(' ').trim();
    }
  }]);

  return CSS;
}();

var hyph = function hyph(s) {
  return s.replace(/[A-Z]|^ms/g, '-$&').toLowerCase();
};
var mx = function mx(rule, media) {
  return media ? media + '{' + rule + '}' : rule;
};
var noAnd = function noAnd(s) {
  return s.replace(/&/g, '');
};
var createDeclaration = function createDeclaration(key, value) {
  return hyph(key) + ':' + value;
};
var createRule = function createRule(_ref) {
  var className = _ref.className,
      child = _ref.child,
      media = _ref.media,
      declarations = _ref.declarations;
  return mx('.' + (className + child) + '{' + declarations.join(';') + '}', media);
};

module.exports = CSS;